@@ -34,6 +34,7 @@ Marks
    Pie
    Map
 """
+from warnings import warn
 from ipywidgets import Widget, DOMWidget, CallbackDispatcher, Color, widget_serialization
 from traitlets import (
         Int, Unicode, List, Enum, Dict, Bool, Float, Instance, Tuple,
@@ -41,9 +42,9 @@ from traitlets import (
     )
 from traittypes import Array
 
-from .scales import Scale, OrdinalScale
+from .scales import Scale, OrdinalScale, LinearScale
 from .traits import Date, array_serialization, array_squeeze, array_dimension_bounds
-
+from ._version import __frontend_version__
 from .colorschemes import CATEGORY10
 
 
@@ -161,6 +162,8 @@ class Mark(Widget):
     _model_name = Unicode('MarkModel').tag(sync=True)
     _model_module = Unicode('bqplot').tag(sync=True)
     _view_module = Unicode('bqplot').tag(sync=True)
+    _view_module_version = Unicode(__frontend_version__).tag(sync=True)
+    _model_module_version = Unicode(__frontend_version__).tag(sync=True)
     _ipython_display_ = None
 
     def _get_dimension_scales(self, dimension, preserve_domain=False):
@@ -348,8 +351,9 @@ class Lines(Mark):
     curves_subset = List().tag(sync=True)
     line_style = Enum(['solid', 'dashed', 'dotted', 'dash_dotted'],
                       default_value='solid').tag(sync=True, display_name='Line style')
-    interpolation = Enum(['linear', 'basis', 'cardinal', 'monotone'], default_value='linear').tag(sync=True,
-                         display_name='Interpolation')
+    interpolation = Enum(['linear', 'basis', 'basis-open', 'basis-closed', 'bundle',
+                          'cardinal', 'cardinal-open', 'cardinal-closed', 'monotone', 'step-before', 'step-after'],
+                         default_value='linear').tag(sync=True, display_name='Interpolation')
     close_path = Bool().tag(sync=True, display_name='Close path')
     fill = Enum(['none', 'bottom', 'top', 'inside'], default_value='none').tag(sync=True, display_name='Fill')
     marker = Enum(['circle', 'cross', 'diamond', 'square', 'triangle-down',
@@ -416,8 +420,67 @@ class FlexLine(Mark):
     _model_name = Unicode('FlexLineModel').tag(sync=True)
 
 
+class _ScatterBase(Mark):
+    """
+    Base Mark for Label and Scatter
+    """
+    # Scaled attributes
+    x = Array([], allow_none=True).tag(sync=True, scaled=True, rtype='Number', atype='bqplot.Axis', **array_serialization).valid(array_dimension_bounds(1, 1))
+    y = Array([], allow_none=True).tag(sync=True, scaled=True, rtype='Number', atype='bqplot.Axis', **array_serialization).valid(array_dimension_bounds(1, 1))
+    color = Array(None, allow_none=True).tag(sync=True, scaled=True, rtype='Color', atype='bqplot.ColorAxis', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
+    opacity = Array(None, allow_none=True).tag(sync=True, scaled=True, rtype='Number', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
+    size = Array(None, allow_none=True).tag(sync=True, scaled=True, rtype='Number', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
+    rotation = Array(None, allow_none=True).tag(sync=True, scaled=True, rtype='Number', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
+
+    # Other attributes
+    scales_metadata = Dict({
+        'x': {'orientation': 'horizontal', 'dimension': 'x'},
+        'y': {'orientation': 'vertical', 'dimension': 'y'},
+        'color': {'dimension': 'color'},
+        'size': {'dimension': 'size'},
+        'opacity': {'dimension': 'opacity'}
+    }).tag(sync=True)
+    default_opacities = List(trait=Float(1.0, min=0, max=1, allow_none=True)).tag(sync=True, display_name='Opacities')
+    hovered_style = Dict().tag(sync=True)
+    unhovered_style = Dict().tag(sync=True)
+    hovered_point = Int(None, allow_none=True).tag(sync=True)
+
+    enable_move = Bool().tag(sync=True)
+    enable_delete = Bool().tag(sync=True)
+    restrict_x = Bool().tag(sync=True)
+    restrict_y = Bool().tag(sync=True)
+    update_on_move = Bool().tag(sync=True)
+
+    def __init__(self, **kwargs):
+        self._drag_start_handlers = CallbackDispatcher()
+        self._drag_handlers = CallbackDispatcher()
+        self._drag_end_handlers = CallbackDispatcher()
+        super(_ScatterBase, self).__init__(**kwargs)
+
+    def on_drag_start(self, callback, remove=False):
+        self._drag_start_handlers.register_callback(callback, remove=remove)
+
+    def on_drag(self, callback, remove=False):
+        self._drag_handlers.register_callback(callback, remove=remove)
+
+    def on_drag_end(self, callback, remove=False):
+        self._drag_end_handlers.register_callback(callback, remove=remove)
+
+    def _handle_custom_msgs(self, _, content, buffers=None):
+        event = content.get('event', '')
+
+        if event == 'drag_start':
+            self._drag_start_handlers(self, content)
+        elif event == 'drag':
+            self._drag_handlers(self, content)
+        elif event == 'drag_end':
+            self._drag_end_handlers(self, content)
+
+        super(_ScatterBase, self)._handle_custom_msgs(self, content)
+
+
 @register_mark('bqplot.Scatter')
-class Scatter(Mark):
+class Scatter(_ScatterBase):
 
     """Scatter mark.
 
@@ -432,11 +495,14 @@ class Scatter(Mark):
         Font-awesome icon for that mark
     name: string (class-level attribute)
         User-friendly name of the mark
-    marker: {'circle', 'cross', 'diamond', 'square', 'triangle-down', 'triangle-up', 'arrow', 'rectangle', 'ellipse'}
+    marker: {'circle', 'cross', 'diamond', 'square', 'triangle-down',
+             'triangle-up', 'arrow', 'rectangle', 'ellipse'}
         Marker shape
-    default_colors: list of colors (default: ['DeepSkyBlue'])
+    colors: list of colors (default: ['DeepSkyBlue'])
         List of colors of the markers. If the list is shorter than the number
         of points, the colors are reused.
+    default_colors: Deprecated
+        Same as `colors`, deprecated as of version 0.8.4
     stroke: Color or None (default: None)
         Stroke color of the marker
     stroke_width: Float (default: 1.5)
@@ -510,34 +576,27 @@ class Scatter(Mark):
     name = 'Scatter'
 
     # Scaled attribtes
-    x = Array([]).tag(sync=True, scaled=True, rtype='Number', atype='bqplot.Axis', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
-    y = Array([]).tag(sync=True, scaled=True, rtype='Number', atype='bqplot.Axis', **array_serialization).valid(array_dimension_bounds(1, 1))
-    color = Array(None, allow_none=True).tag(sync=True, scaled=True, rtype='Color', atype='bqplot.ColorAxis', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
-    opacity = Array(None, allow_none=True).tag(sync=True, scaled=True, rtype='Number', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
-    size = Array(None, allow_none=True).tag(sync=True, scaled=True, rtype='Number', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
     skew = Array(None, allow_none=True).tag(sync=True, scaled=True, rtype='Number', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
-    rotation = Array(None, allow_none=True).tag(sync=True, scaled=True, rtype='Number', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
-    hovered_style = Dict().tag(sync=True)
-    unhovered_style = Dict().tag(sync=True)
-    hovered_point = Int(None, allow_none=True).tag(sync=True)
 
     # Other attributes
-    scales_metadata = Dict({
-        'x': {'orientation': 'horizontal', 'dimension': 'x'},
-        'y': {'orientation': 'vertical', 'dimension': 'y'},
-        'color': {'dimension': 'color'},
-        'size': {'dimension': 'size'},
-        'opacity': {'dimension': 'opacity'}
-    }).tag(sync=True)
     marker = Enum(['circle', 'cross', 'diamond', 'square', 'triangle-down',
                    'triangle-up', 'arrow', 'rectangle', 'ellipse'],
                   default_value='circle').tag(sync=True, display_name='Marker')
-    default_colors = List(trait=Color(default_value=None, allow_none=True),
-                          default_value=['DeepSkyBlue']).tag(sync=True,
-                          display_name='Colors')
+    colors = List(trait=Color(default_value=None, allow_none=True),
+                  default_value=['DeepSkyBlue']).tag(sync=True,
+                  display_name='Colors')
+    @property
+    def default_colors(self):
+        return self.colors
+
+    @default_colors.setter
+    def default_colors(self, value):
+        warn("default_colors is deprecated, use colors instead.",
+             DeprecationWarning)
+        self.colors = value
+
     stroke = Color(None, allow_none=True).tag(sync=True, display_name='Stroke color')
     stroke_width = Float(1.5).tag(sync=True, display_name='Stroke width')
-    default_opacities = List(trait=Float(1.0, min=0, max=1, allow_none=True)).tag(sync=True, display_name='Opacities')
     default_skew = Float(0.5, min=0, max=1).tag(sync=True)
     default_size = Int(64).tag(sync=True, display_name='Default size')
 
@@ -548,43 +607,80 @@ class Scatter(Mark):
     drag_size = Float(5.).tag(sync=True)
     names_unique = Bool(True).tag(sync=True)
 
-    enable_move = Bool().tag(sync=True)
-    enable_delete = Bool().tag(sync=True)
-    restrict_x = Bool().tag(sync=True)
-    restrict_y = Bool().tag(sync=True)
-    update_on_move = Bool().tag(sync=True)
-
-    def __init__(self, **kwargs):
-        self._drag_start_handlers = CallbackDispatcher()
-        self._drag_handlers = CallbackDispatcher()
-        self._drag_end_handlers = CallbackDispatcher()
-        super(Scatter, self).__init__(**kwargs)
-
-    def on_drag_start(self, callback, remove=False):
-        self._drag_start_handlers.register_callback(callback, remove=remove)
-
-    def on_drag(self, callback, remove=False):
-        self._drag_handlers.register_callback(callback, remove=remove)
-
-    def on_drag_end(self, callback, remove=False):
-        self._drag_end_handlers.register_callback(callback, remove=remove)
-
-    def _handle_custom_msgs(self, _, content, buffers=None):
-        event = content.get('event', '')
-
-        if event == 'drag_start':
-            self._drag_start_handlers(self, content)
-        elif event == 'drag':
-            self._drag_handlers(self, content)
-        elif event == 'drag_end':
-            self._drag_end_handlers(self, content)
-
-        super(Scatter, self)._handle_custom_msgs(self, content)
-
     _view_name = Unicode('Scatter').tag(sync=True)
     _model_name = Unicode('ScatterModel').tag(sync=True)
 
 
+@register_mark('bqplot.Label')
+class Label(_ScatterBase):
+
+    """Label mark.
+
+    Attributes
+    ----------
+    x_offset: int (default: 0)
+        horizontal offset in pixels from the stated x location
+    y_offset: int (default: 0)
+        vertical offset in pixels from the stated y location
+    text: string (default: '')
+        text to be displayed
+    default_size: string (default: '14px')
+        font size in px, em or ex
+    font_weight: {'bold', 'normal', 'bolder'}
+        font weight of the caption
+    drag_size: nonnegative float (default: 1.)
+        Ratio of the size of the dragged label font size to the default label font size.
+    align: {'start', 'middle', 'end'}
+        alignment of the text with respect to the provided location
+        enable_move: Bool (default: False)
+        Enable the label to be moved by dragging. Refer to restrict_x,
+        restrict_y for more options.
+    restrict_x: bool (default: False)
+        Restricts movement of the label to only along the x axis. This is valid
+        only when enable_move is set to True. If both restrict_x and restrict_y
+        are set to True, the label cannot be moved.
+    restrict_y: bool (default: False)
+        Restricts movement of the label to only along the y axis. This is valid
+        only when enable_move is set to True. If both restrict_x and restrict_y
+        are set to True, the label cannot be moved.
+
+    Data Attributes
+
+    x: numpy.ndarray (default: [])
+        horizontal position of the labels, in data coordinates or in figure coordinates
+    y: numpy.ndarray (default: [])
+        vertical position of the labels, in data coordinates or in figure coordinates
+    color: numpy.ndarray or None (default: None)
+        label colors
+    size: numpy.ndarray or None (default: None)
+        label sizes
+    rotation: numpy.ndarray or None (default: None)
+        label rotations
+    opacity: numpy.ndarray or None (default: None)
+        label opacities
+
+    """
+    # Mark decoration
+    icon = 'fa-font'
+    name = 'Labels'
+
+    # Other attributes
+    x_offset = Int().tag(sync=True)
+    y_offset = Int().tag(sync=True)
+
+    colors = List(trait=Color(default_value=None, allow_none=True), default_value=CATEGORY10).tag(sync=True, display_name='Colors')
+    rotate_angle = Float().tag(sync=True)
+    text = Array(None, allow_none=True).tag(sync=True, **array_serialization).valid(array_squeeze)
+    default_size = Float(16.).tag(sync=True)
+    drag_size = Float(1.).tag(sync=True)
+    font_unit = Enum(['px', 'em', 'pt', '%'], default_value='px').tag(sync=True)
+    font_weight = Enum(['bold', 'normal', 'bolder'], default_value='bold').tag(sync=True)
+    align = Enum(['start', 'middle', 'end'], default_value='start').tag(sync=True)
+
+    _view_name = Unicode('Label').tag(sync=True)
+    _model_name = Unicode('LabelModel').tag(sync=True)
+
+
 @register_mark('bqplot.Hist')
 class Hist(Mark):
 
@@ -736,6 +832,9 @@ class Bars(Mark):
         whether 2-dimensional bar charts should appear grouped or stacked.
     colors: list of colors (default: CATEGORY10)
         list of colors for the bars.
+    orientation: {'horizontal', 'vertical'}
+        Specifies whether the bar chart is drawn horizontally or vertically.
+        If a horizontal bar chart is drawn, the x data is drawn vertically.
     padding: float (default: 0.05)
         attribute to control the spacing between the bars value is specified
         as a percentage of the width of the bar
@@ -790,130 +889,20 @@ class Bars(Mark):
     base = Float().tag(sync=True)
     opacities = List(trait=Float(1.0, min=0, max=1, allow_none=True)).tag(sync=True, display_name='Opacities')
     align = Enum(['center', 'left', 'right'], default_value='center').tag(sync=True)
+    orientation = Enum(['vertical', 'horizontal'], default_value='vertical').tag(sync=True)
+
+    @validate('orientation')
+    def _validate_orientation(self, proposal):
+        value = proposal['value']
+        x_orient = "horizontal" if value == "vertical" else "vertical"
+        self.scales_metadata = {'x': {'orientation': x_orient, 'dimension': 'x'},
+                                'y': {'orientation': value, 'dimension': 'y'}}
+        return value
 
     _view_name = Unicode('Bars').tag(sync=True)
     _model_name = Unicode('BarsModel').tag(sync=True)
 
 
-@register_mark('bqplot.Label')
-class Label(Mark):
-
-    """Label mark.
-
-    Attributes
-    ----------
-    x_offset: int (default: 0)
-        horizontal offset in pixels from the stated x location
-    y_offset: int (default: 0)
-        vertical offset in pixels from the stated y location
-    text: string (default: '')
-        text to be displayed
-    font_size: string (default: '14px')
-        font size in px, em or ex
-    font_weight: {'bold', 'normal', 'bolder'}
-        font weight of the caption
-    drag_size: nonnegative float (default: 1.)
-        Ratio of the size of the dragged label font size to the default label font size.
-    align: {'start', 'middle', 'end'}
-        alignment of the text with respect to the provided location
-    enable_move: Bool (default: False)
-        Enable the label to be moved by dragging. Refer to restrict_x,
-        restrict_y for more options.
-    restrict_x: bool (default: False)
-        Restricts movement of the label to only along the x axis. This is valid
-        only when enable_move is set to True. If both restrict_x and restrict_y
-        are set to True, the label cannot be moved.
-    restrict_y: bool (default: False)
-        Restricts movement of the label to only along the y axis. This is valid
-        only when enable_move is set to True. If both restrict_x and restrict_y
-        are set to True, the label cannot be moved.
-
-    Data Attributes
-
-    x: numpy.ndarray (default: [])
-        horizontal position of the labels, in data coordinates or in figure coordinates
-    y: numpy.ndarray (default: [])
-        vertical position of the labels, in data coordinates or in figure coordinates
-    color: numpy.ndarray or None (default: None)
-        label colors
-    size: numpy.ndarray or None (default: None)
-        label sizes
-    rotation: numpy.ndarray or None (default: None)
-        label rotations
-    opacity: numpy.ndarray or None (default: None)
-        label opacities
-
-    """
-    # Mark decoration
-    icon = 'fa-font'
-    name = 'Labels'
-
-    # Scaled attributes
-    x = Array(None, allow_none=True).tag(sync=True, scaled=True, rtype='Number', atype='bqplot.Axis', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
-    y = Array(None, allow_none=True).tag(sync=True, scaled=True, rtype='Number', atype='bqplot.Axis', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
-    color = Array(None, allow_none=True).tag(sync=True, scaled=True,
-                rtype='Color', atype='bqplot.ColorAxis', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
-    size = Array(None, allow_none=True).tag(sync=True, scaled=True,
-                   rtype='Number', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
-    rotation = Array(None, allow_none=True).tag(sync=True, scaled=True,
-                       rtype='Number', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
-    opacity = Array(None, allow_none=True).tag(sync=True, scaled=True,
-                      rtype='Number', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
-
-    # Other attributes
-    x_offset = Int().tag(sync=True)
-    y_offset = Int().tag(sync=True)
-    scales_metadata = Dict({
-        'x': { 'orientation': 'horizontal', 'dimension': 'x' },
-        'y': { 'orientation': 'vertical', 'dimension': 'y' },
-        'color': { 'dimension': 'color' }
-    }).tag(sync=True)
-    colors = List(trait=Color(default_value=None, allow_none=True), default_value=CATEGORY10).tag(sync=True, display_name='Colors')
-    default_opacities = List(trait=Float(1.0, min=0, max=1, allow_none=True)).tag(sync=True, display_name='Opacities')
-    rotate_angle = Float().tag(sync=True)
-    text = Array(None, allow_none=True).tag(sync=True, **array_serialization).valid(array_squeeze)
-    font_size = Float(16.).tag(sync=True)
-    drag_size = Float(1.).tag(sync=True)
-    font_unit = Enum(['px', 'em', 'pt', '%'], default_value='px').tag(sync=True)
-    font_weight = Enum(['bold', 'normal', 'bolder'], default_value='bold').tag(sync=True)
-    align = Enum(['start', 'middle', 'end'], default_value='start').tag(sync=True)
-
-    enable_move = Bool().tag(sync=True)
-    restrict_x = Bool().tag(sync=True)
-    restrict_y = Bool().tag(sync=True)
-    update_on_move = Bool().tag(sync=True)
-
-    def __init__(self, **kwargs):
-        self._drag_start_handlers = CallbackDispatcher()
-        self._drag_handlers = CallbackDispatcher()
-        self._drag_end_handlers = CallbackDispatcher()
-        super(Label, self).__init__(**kwargs)
-
-    def on_drag_start(self, callback, remove=False):
-        self._drag_start_handlers.register_callback(callback, remove=remove)
-
-    def on_drag(self, callback, remove=False):
-        self._drag_handlers.register_callback(callback, remove=remove)
-
-    def on_drag_end(self, callback, remove=False):
-        self._drag_end_handlers.register_callback(callback, remove=remove)
-
-    def _handle_custom_msgs(self, _, content, buffers=None):
-        event = content.get('event', '')
-
-        if event == 'drag_start':
-            self._drag_start_handlers(self, content)
-        elif event == 'drag':
-            self._drag_handlers(self, content)
-        elif event == 'drag_end':
-            self._drag_end_handlers(self, content)
-
-        super(Label, self)._handle_custom_msgs(self, content)
-
-    _view_name = Unicode('Label').tag(sync=True)
-    _model_name = Unicode('LabelModel').tag(sync=True)
-
-
 @register_mark('bqplot.OHLC')
 class OHLC(Mark):
 
@@ -1183,7 +1172,7 @@ class GridHeatMap(Mark):
         `start` aligns the column values passed to be aligned with the start of the
         tiles and `end` aligns the column values to the end of the tiles.
     anchor_style: dict (default: {'fill': 'white', 'stroke': 'blue'})
-        Controls the style for the element which serves as the anchor duringi
+        Controls the style for the element which serves as the anchor during
         selection.
 
     Data Attributes
@@ -1197,7 +1186,7 @@ class GridHeatMap(Mark):
         This is a scaled attribute and can be used to affect the height of the
         cells as the entries of `row` can indicate the start or the end points
         of the cells. Refer to the property `row_align`.
-        If this prorety is None, then a uniformly spaced grid is generated in
+        If this property is None, then a uniformly spaced grid is generated in
         the row direction.
     column: numpy.ndarray or None (default: None)
         labels for the columns of the `color` array passed. The length of this can be
@@ -1205,7 +1194,7 @@ class GridHeatMap(Mark):
         This is a scaled attribute and can be used to affect the width of the
         cells as the entries of `column` can indicate the start or the end points
         of the cells. Refer to the property `column_align`.
-        If this prorety is None, then a uniformly spaced grid is generated in
+        If this property is None, then a uniformly spaced grid is generated in
         the column direction.
     """
     # Scaled attributes
@@ -1232,18 +1221,16 @@ class GridHeatMap(Mark):
 
     def __init__(self, **kwargs):
         data = kwargs['color']
-        row = kwargs.pop('row', range(data.shape[0]))
-        column = kwargs.pop('column', range(data.shape[1]))
+        kwargs.setdefault('row', range(data.shape[0]))
+        kwargs.setdefault('column', range(data.shape[1]))
         scales = kwargs.pop('scales', {})
         # Adding default row and column data if they are not passed.
         # Adding scales in case they are not passed too.
 
-        kwargs['row'] = row
         if(scales.get('row', None) is None):
             row_scale = OrdinalScale(reverse=True)
             scales['row'] = row_scale
 
-        kwargs['column'] = column
         if(scales.get('column', None) is None):
             column_scale = OrdinalScale()
             scales['column'] = column_scale
@@ -1252,3 +1239,64 @@ class GridHeatMap(Mark):
 
     _view_name = Unicode('GridHeatMap').tag(sync=True)
     _model_name = Unicode('GridHeatMapModel').tag(sync=True)
+
+
+@register_mark('bqplot.HeatMap')
+class HeatMap(Mark):
+
+    """HeatMap mark.
+
+
+    Attributes
+    ----------
+
+    Data Attributes
+
+    color: numpy.ndarray or None (default: None)
+        color of the data points (2d array).
+    x: numpy.ndarray or None (default: None)
+        labels for the columns of the `color` array passed. The length of this has
+        to be the number of columns in `color`.
+        This is a scaled attribute.
+    y: numpy.ndarray or None (default: None)
+        labels for the rows of the `color` array passed. The length of this has
+        to be the number of rows in `color`.
+        This is a scaled attribute.
+    """
+    # Scaled attributes
+    x = Array(None, allow_none=True).tag(sync=True, scaled=True,
+                  rtype='Number', atype='bqplot.Axis', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
+    y = Array(None, allow_none=True).tag(sync=True, scaled=True,
+                     rtype='Number', atype='bqplot.Axis', **array_serialization).valid(array_squeeze, array_dimension_bounds(1, 1))
+    color = Array(None, allow_none=True).tag(sync=True, scaled=True, rtype='Color',
+                    atype='bqplot.ColorAxis', **array_serialization).valid(array_squeeze, array_dimension_bounds(2, 2))
+
+    # Other attributes
+    scales_metadata = Dict({
+        'x': {'orientation': 'horizontal', 'dimension': 'x'},
+        'y': {'orientation': 'vertical', 'dimension': 'y'},
+        'color': {'dimension': 'color'}
+    }).tag(sync=True)
+
+    null_color = Color('black', allow_none=True).tag(sync=True)
+
+    def __init__(self, **kwargs):
+        data = kwargs['color']
+        kwargs.setdefault('x', range(data.shape[1]))
+        kwargs.setdefault('y', range(data.shape[0]))
+        scales = kwargs.pop('scales', {})
+        # Adding default x and y data if they are not passed.
+        # Adding scales in case they are not passed too.
+
+        if(scales.get('x', None) is None):
+            x_scale = LinearScale()
+            scales['x'] = x_scale
+
+        if(scales.get('y', None) is None):
+            y_scale = LinearScale()
+            scales['y'] = y_scale
+        kwargs['scales'] = scales
+        super(HeatMap, self).__init__(**kwargs)
+
+    _view_name = Unicode('HeatMap').tag(sync=True)
+    _model_name = Unicode('HeatMapModel').tag(sync=True)
