@@ -43,14 +43,18 @@ Pyplot
    ylim
 
 """
+import sys
 from collections import OrderedDict
 from IPython.display import display
 from ipywidgets import VBox
-from numpy import arange, issubdtype, array, column_stack
+from numpy import arange, issubdtype, array, column_stack, shape
 from .figure import Figure
 from .scales import Scale, LinearScale, Mercator
 from .axes import Axis
-from .marks import Lines, Scatter, Hist, Bars, OHLC, Pie, Map, Label
+from .marks import (
+        Lines, Scatter, Hist, Bars, OHLC, Pie, Map,
+        Label, HeatMap, topo_load
+    )
 from .toolbar import Toolbar
 from .interacts import (
         BrushIntervalSelector, FastIntervalSelector, BrushSelector,
@@ -92,6 +96,23 @@ MARKER_CODES = {'o': 'circle', 'v': 'triangle-down', '^': 'triangle-up',
                 's': 'square', 'd': 'diamond', '+': 'cross'}
 
 
+PY2 = sys.version_info[0] == 2
+
+if PY2:
+    string_types = basestring,
+else:
+    string_types = str,
+
+
+def hashable(data, v):
+    """Determine whether `v` can be hashed."""
+    try:
+        data[v]
+    except (TypeError, KeyError, IndexError):
+        return False
+    return True
+
+
 def show(key=None, display_toolbar=True):
     """Shows the current context figure in the output area.
 
@@ -211,6 +232,27 @@ def close(key):
     del _context['scale_registry'][key]
 
 
+def _process_data(*kwarg_names):
+    """Helper function to handle data keyword argument
+    """
+    def _data_decorator(func):
+        def _mark_with_data(*args, **kwargs):
+            data = kwargs.pop('data', None)
+            if data is None:
+                return func(*args, **kwargs)
+            else:
+                data_args = [data[i] if hashable(data, i) else i for i in args]
+                data_kwargs = {
+                   kw: data[kwargs[kw]] if hashable(data, kwargs[kw]) else kwargs[kw] for kw in set(kwarg_names).intersection(list(kwargs.keys()))
+                }
+                kwargs_update = kwargs.copy()
+                kwargs_update.update(data_kwargs)
+                return func(*data_args, **kwargs_update)
+
+        return _mark_with_data
+    return _data_decorator
+
+
 def scales(key=None, scales={}):
     """Creates and switches between context scales.
 
@@ -393,24 +435,21 @@ def legend():
         m.display_legend = True
 
 
-def hline(level, fig=None, **kwargs):
+def hline(level, **kwargs):
     """Draws a horizontal line at the given level.
 
     Parameters
     ----------
     level: float
         The level at which to draw the horizontal line.
-    fig: Figure or None
-        The figure to which the horizontal line is to be added.
-        If the value is None, the current figure is used.
     preserve_domain: boolean (default: False)
         If true, the line does not affect the domain of the 'y' scale.
     """
-    default_colors = kwargs.pop('colors', ['dodgerblue'])
-    default_width = kwargs.pop('stroke_width', 1)
-    if fig is None:
-        fig = current_figure()
-    sc_x = fig.scale_x
+    kwargs.setdefault('colors', ['dodgerblue'])
+    kwargs.setdefault('stroke_width', 1)
+    scales = kwargs.pop('scales', {})
+    fig = kwargs.get('figure', current_figure())
+    scales['x'] = fig.scale_x
 
     level = array(level)
     if len(level.shape) == 0:
@@ -419,30 +458,27 @@ def hline(level, fig=None, **kwargs):
     else:
         x = [0, 1]
         y = column_stack([level, level])
-    return plot(x, y, scales={ 'x': sc_x }, preserve_domain={
+    return plot(x, y, scales=scales, preserve_domain={
         'x': True,
         'y': kwargs.get('preserve_domain', False)
-    }, axes=False, colors=default_colors, stroke_width=default_width, update_context=False, **kwargs)
+    }, axes=False, update_context=False, **kwargs)
 
 
-def vline(level, fig=None, **kwargs):
+def vline(level, **kwargs):
     """Draws a vertical line at the given level.
 
     Parameters
     ----------
     level: float
         The level at which to draw the vertical line.
-    fig: Figure or None
-        The figure to which the vertical line is to be added.
-        If the value is None, the current figure is used.
     preserve_domain: boolean (default: False)
         If true, the line does not affect the domain of the 'x' scale.
     """
-    default_colors = kwargs.pop('colors', ['dodgerblue'])
-    default_width = kwargs.pop('stroke_width', 1)
-    if fig is None:
-        fig = current_figure()
-    sc_y = fig.scale_y
+    kwargs.setdefault('colors', ['dodgerblue'])
+    kwargs.setdefault('stroke_width', 1)
+    scales = kwargs.pop('scales', {})
+    fig = kwargs.get('figure', current_figure())
+    scales['y'] = fig.scale_y
 
     level = array(level)
     if len(level.shape) == 0:
@@ -452,10 +488,10 @@ def vline(level, fig=None, **kwargs):
         x = column_stack([level, level])
         y = [[0, 1]] * len(level)  # TODO: repeating [0, 1] should not be
                                    # required once we allow for 2-D x and 1-D y
-    return plot(x, y, scales={ 'y': sc_y }, preserve_domain={
+    return plot(x, y, scales=scales, preserve_domain={
         'x': kwargs.get('preserve_domain', False),
         'y': True
-    }, axes=False, colors=default_colors, stroke_width=default_width, update_context=False, **kwargs)
+    }, axes=False, update_context=False, **kwargs)
 
 
 def _draw_mark(mark_type, options={}, axes_options={}, **kwargs):
@@ -473,6 +509,9 @@ def _draw_mark(mark_type, options={}, axes_options={}, **kwargs):
         Options for the axes to be created. If an axis labeled 'x' is required
         for that mark, axes_options['x'] contains optional keyword arguments
         for the constructor of the corresponding axis type.
+    figure: Figure or None
+        The figure to which the mark is to be added.
+        If the value is None, the current figure is used.
     """
     fig = kwargs.pop('figure', current_figure())
     scales = kwargs.pop('scales', {})
@@ -508,7 +547,7 @@ def _draw_mark(mark_type, options={}, axes_options={}, **kwargs):
                 ]
             sorted_scales = sorted(compat_scale_types, key=lambda x: x.precedence)
             scales[name] = sorted_scales[-1](**options.get(name, {}))
-            # Adding the scale to the conext scales
+            # Adding the scale to the context scales
             if update_context:
                 _context['scales'][dimension] = scales[name]
         else:
@@ -521,7 +560,21 @@ def _draw_mark(mark_type, options={}, axes_options={}, **kwargs):
         axes(mark, options=axes_options)
     return mark
 
+def _infer_x_for_line(y):
+    """
+    Infers the x for a line if no x is provided.
+    """
+    array_shape = shape(y)
 
+    if len(array_shape) == 0:
+        return []
+    if len(array_shape) == 1:
+        return arange(array_shape[0])
+    if len(array_shape) > 1:
+        return arange(array_shape[1])
+
+
+@_process_data('color')
 def plot(*args, **kwargs):
     """Draw lines in the current context figure.
 
@@ -549,16 +602,19 @@ def plot(*args, **kwargs):
         Options for the axes to be created. If an axis labeled 'x' is required
         for that mark, axes_options['x'] contains optional keyword arguments
         for the constructor of the corresponding axis type.
+    figure: Figure or None
+        The figure to which the line is to be added.
+        If the value is None, the current figure is used.
     """
     marker_str = None
 
     if len(args) == 1:
         kwargs['y'] = args[0]
-        kwargs['x'] = arange(len(args[0]))
+        kwargs['x'] = _infer_x_for_line(args[0])
     elif len(args) == 2:
         if type(args[1]) == str:
             kwargs['y'] = args[0]
-            kwargs['x'] = arange(len(args[0]))
+            kwargs['x'] = _infer_x_for_line(args[0])
             marker_str = args[1].strip()
         else:
             kwargs['x'] = args[0]
@@ -624,6 +680,7 @@ def ohlc(*args, **kwargs):
     return _draw_mark(OHLC, **kwargs)
 
 
+@_process_data('color', 'opacity', 'size', 'skew', 'rotation')
 def scatter(x, y, **kwargs):
     """Draw a scatter in the current context figure.
 
@@ -648,6 +705,7 @@ def scatter(x, y, **kwargs):
     return _draw_mark(Scatter, **kwargs)
 
 
+@_process_data()
 def hist(sample, options={}, **kwargs):
     """Draw a histogram in the current context figure.
 
@@ -677,6 +735,7 @@ def hist(sample, options={}, **kwargs):
     return _draw_mark(Hist, options=options, **kwargs)
 
 
+@_process_data('color')
 def bar(x, y, **kwargs):
     """Draws a bar chart in the current context figure.
 
@@ -701,6 +760,31 @@ def bar(x, y, **kwargs):
     return _draw_mark(Bars, **kwargs)
 
 
+@_process_data('color')
+def barh(*args, **kwargs):
+    """Draws a horizontal bar chart in the current context figure.
+
+    Parameters
+    ----------
+
+    x: numpy.ndarray, 1d
+        The domain of the data points.
+    y: numpy.ndarray, 1d
+        The range of the data pints.
+    options: dict (default: {})
+        Options for the scales to be created. If a scale labeled 'x' is
+        required for that mark, options['x'] contains optional keyword
+        arguments for the constructor of the corresponding scale type.
+    axes_options: dict (default: {})
+        Options for the axes to be created. If an axis labeled 'x' is required
+        for that mark, axes_options['x'] contains optional keyword arguments
+        for the constructor of the corresponding axis type.
+    """
+    kwargs['orientation'] = "horizontal"
+    return bar(*args, **kwargs)
+
+
+@_process_data('color')
 def pie(sizes, **kwargs):
     """Draws a Pie in the current context figure.
 
@@ -747,6 +831,8 @@ def geo(map_data, **kwargs):
 
     Parameters
     ----------
+    map_data: string or bqplot.map (default: WorldMap)
+        Name of the map or json file required for the map data.
     options: dict (default: {})
         Options for the scales to be created. If a scale labeled 'x' is
         required for that mark, options['x'] contains optional keyword
@@ -761,10 +847,32 @@ def geo(map_data, **kwargs):
     if 'projection' not in scales:
         scales['projection'] = Mercator(**options.get('projection', {}))
     kwargs['scales'] = scales
-    kwargs['map_data'] = map_data
+    if isinstance(map_data, string_types):
+        kwargs['map_data'] = topo_load('map_data/' + map_data + '.json')
+    else:
+        kwargs['map_data'] = map_data
     return _draw_mark(Map, **kwargs)
 
 
+def heatmap(color, **kwargs):
+    """Draw a heatmap in the current context figure.
+
+    Parameters
+    ----------
+    color: numpy.ndarray, 2d
+        Matrix of color of the data points
+    options: dict (default: {})
+        Options for the scales to be created. If a scale labeled 'x' is
+        required for that mark, options['x'] contains optional keyword
+        arguments for the constructor of the corresponding scale type.
+    axes_options: dict (default: {})
+        Options for the axes to be created. If an axis labeled 'x' is required
+        for that mark, axes_options['x'] contains optional keyword arguments
+        for the constructor of the corresponding axis type.
+    """
+    kwargs['color'] = color
+    return _draw_mark(HeatMap, **kwargs)
+
 def _add_interaction(int_type, **kwargs):
     """Add the interaction for the specified type.
 
